from ..configs import mobile_models
from random import choice
from string import ascii_lowercase, digits
from hashlib import md5

class Utils:
    def generateRandomPassport():
        return "".join(choice(ascii_lowercase + digits) for _ in range(32))
    def generateRandomUdid():
        return "".join(choice(ascii_lowercase + digits) for _ in range(16))
    def getRandomMobileModel():
        return choice(mobile_models)
    def hashQueueNumber(q: int):
        return md5(str(q).encode()).hexdigest()